@@ -20,16 +20,17 @@
 package org.xnio;
 
 import java.io.IOException;
+import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.net.SocketAddress;
 import java.security.PrivilegedAction;
+import java.util.Collections;
 import java.util.List;
 import java.util.Set;
 import java.util.concurrent.AbstractExecutorService;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.ExecutorService;
-import java.util.concurrent.LinkedBlockingQueue;
-import java.util.concurrent.RejectedExecutionHandler;
+import java.util.concurrent.LinkedBlockingDeque;
 import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
@@ -37,6 +38,14 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
 import java.util.zip.Deflater;
 import java.util.zip.Inflater;
+
+import org.jboss.threads.EnhancedQueueExecutor;
+import org.wildfly.common.Assert;
+import org.wildfly.common.context.ContextManager;
+import org.wildfly.common.context.Contextual;
+import org.wildfly.common.net.CidrAddress;
+import org.wildfly.common.net.CidrAddressTable;
+import org.xnio._private.Messages;
 import org.xnio.channels.AcceptingChannel;
 import org.xnio.channels.AssembledConnectedMessageChannel;
 import org.xnio.channels.AssembledConnectedStreamChannel;
@@ -54,8 +63,12 @@ import org.xnio.conduits.DeflatingStreamSinkConduit;
 import org.xnio.conduits.InflatingStreamSourceConduit;
 import org.xnio.conduits.StreamSinkChannelWrappingConduit;
 import org.xnio.conduits.StreamSourceChannelWrappingConduit;
+import org.xnio.management.XnioServerMXBean;
+import org.xnio.management.XnioWorkerMXBean;
 
+import static java.lang.Math.max;
 import static java.security.AccessController.doPrivileged;
+import org.jboss.logging.Logger;
 import static org.xnio.IoUtils.safeClose;
 import static org.xnio._private.Messages.msg;
 
@@ -67,60 +80,109 @@ import static org.xnio._private.Messages.msg;
  * @since 3.0
  */
 @SuppressWarnings("unused")
-public abstract class XnioWorker extends AbstractExecutorService implements Configurable, ExecutorService, XnioIoFactory {
+public abstract class XnioWorker extends AbstractExecutorService implements Configurable, ExecutorService, XnioIoFactory, Contextual<XnioWorker> {
 
     private final Xnio xnio;
     private final TaskPool taskPool;
     private final String name;
     private final Runnable terminationTask;
+    private final CidrAddressTable<InetSocketAddress> bindAddressTable;
 
     private volatile int taskSeq;
-    private volatile int coreSize;
 
     private static final AtomicIntegerFieldUpdater<XnioWorker> taskSeqUpdater = AtomicIntegerFieldUpdater.newUpdater(XnioWorker.class, "taskSeq");
-    private static final AtomicIntegerFieldUpdater<XnioWorker> coreSizeUpdater = AtomicIntegerFieldUpdater.newUpdater(XnioWorker.class, "coreSize");
 
     private static final AtomicInteger seq = new AtomicInteger(1);
 
     private static final RuntimePermission CREATE_WORKER_PERMISSION = new RuntimePermission("createXnioWorker");
-    private final BlockingQueue<Runnable> taskQueue;
 
     private int getNextSeq() {
         return taskSeqUpdater.incrementAndGet(this);
     }
 
+    private static final Logger log = Logger.getLogger("org.xnio");
+
     /**
-     * Construct a new instance.  Intended to be called only from implementations.  To construct an XNIO worker,
-     * use the {@link Xnio#createWorker(OptionMap)} method.
+     * Construct a new instance.  Intended to be called only from implementations.
      *
-     * @param xnio the XNIO provider which produced this worker instance
-     * @param threadGroup the thread group for worker threads
-     * @param optionMap the option map to use to configure this worker
-     * @param terminationTask an optional runnable task to run when the worker shutdown completes
+     * @param builder the worker builder
      */
-    protected XnioWorker(final Xnio xnio, final ThreadGroup threadGroup, final OptionMap optionMap, final Runnable terminationTask) {
-        this.xnio = xnio;
-        this.terminationTask = terminationTask;
+    protected XnioWorker(final Builder builder) {
+        this.xnio = builder.xnio;
+        this.terminationTask = builder.terminationTask;
         final SecurityManager sm = System.getSecurityManager();
         if (sm != null) {
             sm.checkPermission(CREATE_WORKER_PERMISSION);
         }
-        String workerName = optionMap.get(Options.WORKER_NAME);
+        String workerName = builder.getWorkerName();
         if (workerName == null) {
             workerName = "XNIO-" + seq.getAndIncrement();
         }
         name = workerName;
-        taskQueue = new LinkedBlockingQueue<Runnable>();
-        this.coreSize = optionMap.get(Options.WORKER_TASK_CORE_THREADS, 4);
-        final boolean markThreadAsDaemon = optionMap.get(Options.THREAD_DAEMON, false);
-        final int threadCount = optionMap.get(Options.WORKER_TASK_MAX_THREADS, 16);
-        taskPool = new TaskPool(
-            threadCount, // ignore core threads setting, always fill to max
-            threadCount,
-            optionMap.get(Options.WORKER_TASK_KEEPALIVE, 60000), TimeUnit.MILLISECONDS,
-            taskQueue,
-            new WorkerThreadFactory(threadGroup, optionMap, markThreadAsDaemon),
-            new ThreadPoolExecutor.AbortPolicy());
+        final boolean markThreadAsDaemon = builder.isDaemon();
+        bindAddressTable = builder.getBindAddressConfigurations();
+        final Runnable terminationTask = new Runnable() {
+            public void run() {
+                taskPoolTerminated();
+            }
+        };
+        final ExecutorService executorService = builder.getExternalExecutorService();
+        if (executorService != null) {
+            taskPool = new ExternalTaskPool(executorService);
+        } else if (EnhancedQueueExecutor.DISABLE_HINT) {
+            final int poolSize = max(builder.getMaxWorkerPoolSize(), builder.getCoreWorkerPoolSize());
+            taskPool = new ThreadPoolExecutorTaskPool(
+                poolSize,
+                poolSize,
+                builder.getWorkerKeepAlive(), TimeUnit.MILLISECONDS,
+                new LinkedBlockingDeque<>(),
+                new WorkerThreadFactory(builder.getThreadGroup(), builder.getWorkerStackSize(), markThreadAsDaemon),
+                terminationTask);
+        } else {
+            taskPool = new EnhancedQueueExecutorTaskPool(new EnhancedQueueExecutor.Builder()
+                .setCorePoolSize(builder.getCoreWorkerPoolSize())
+                .setMaximumPoolSize(builder.getMaxWorkerPoolSize())
+                .setKeepAliveTime(builder.getWorkerKeepAlive(), TimeUnit.MILLISECONDS)
+                .setThreadFactory(new WorkerThreadFactory(builder.getThreadGroup(), builder.getWorkerStackSize(), markThreadAsDaemon))
+                .setTerminationTask(terminationTask)
+                .setRegisterMBean(true)
+                .setMBeanName(workerName)
+                .build()
+            );
+        }
+    }
+
+    //==================================================
+    //
+    // Context methods
+    //
+    //==================================================
+
+    private static final ContextManager<XnioWorker> CONTEXT_MANAGER = doPrivileged((PrivilegedAction<ContextManager<XnioWorker>>) () -> new ContextManager<XnioWorker>(XnioWorker.class, "org.xnio.worker"));
+
+    static {
+        doPrivileged((PrivilegedAction<Void>) () -> {
+            CONTEXT_MANAGER.setGlobalDefaultSupplier(() -> DefaultXnioWorkerHolder.INSTANCE);
+            return null;
+        });
+    }
+
+    /**
+     * Get the context manager for XNIO workers.
+     *
+     * @return the context manager (not {@code null})
+     */
+    public static ContextManager<XnioWorker> getContextManager() {
+        return CONTEXT_MANAGER;
+    }
+
+    /**
+     * Get the instance context manager for XNIO workers by delegating to {@link #getContextManager()}.
+     *
+     * @return the context manager (not {@code null})
+     */
+    public ContextManager<XnioWorker> getInstanceContextManager() {
+        return getContextManager();
     }
 
     //==================================================
@@ -199,7 +261,7 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
             }
 
             public XnioIoThread getIoThread() {
-                return chooseThread();
+                return server.getIoThread();
             }
 
             public void close() throws IOException {
@@ -236,9 +298,7 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
      * @throws IOException if the server could not be created
      */
     public AcceptingChannel<StreamConnection> createStreamConnectionServer(SocketAddress bindAddress, ChannelListener<? super AcceptingChannel<StreamConnection>> acceptListener, OptionMap optionMap) throws IOException {
-        if (bindAddress == null) {
-            throw msg.nullParameter("bindAddress");
-        }
+        Assert.checkNotNullParam("bindAddress", bindAddress);
         if (bindAddress instanceof InetSocketAddress) {
             return createTcpConnectionServer((InetSocketAddress) bindAddress, acceptListener, optionMap);
         } else if (bindAddress instanceof LocalSocketAddress) {
@@ -701,6 +761,14 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
         return chooseThread();
     }
 
+    /**
+     * Get an I/O thread from this worker.  The thread is chosen based on the given hash code.
+     *
+     * @param hashCode the hash code
+     * @return the thread
+     */
+    public abstract XnioIoThread getIoThread(int hashCode);
+
     /**
      * Get the user task to run once termination is complete.
      *
@@ -711,7 +779,7 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
     }
 
     /**
-     * Callback to indicate that the task thread pool has terminated.
+     * Callback to indicate that the task thread pool has terminated.  Not called if the task pool is external.
      */
     protected void taskPoolTerminated() {}
 
@@ -720,7 +788,16 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
      * the {@link #taskPoolTerminated()} method is called.
      */
     protected void shutDownTaskPool() {
-        taskPool.shutdown();
+        if (isTaskPoolExternal()) {
+            taskPoolTerminated();
+        } else {
+            doPrivileged(new PrivilegedAction<Object>() {
+                public Object run() {
+                    taskPool.shutdown();
+                    return null;
+                }
+            });
+        }
     }
 
     /**
@@ -729,7 +806,22 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
      * @return the pending task list
      */
     protected List<Runnable> shutDownTaskPoolNow() {
-        return taskPool.shutdownNow();
+        if (! isTaskPoolExternal()) return doPrivileged(new PrivilegedAction<List<Runnable>>() {
+            public List<Runnable> run() {
+                return taskPool.shutdownNow();
+            }
+        });
+        return Collections.emptyList();
+    }
+
+    /**
+     * Determine whether the worker task pool is managed externally.  Externally managed task pools will never
+     * respond to shut down requests.
+     *
+     * @return {@code true} if the task pool is externally managed, {@code false} otherwise
+     */
+    protected boolean isTaskPoolExternal() {
+        return taskPool instanceof ExternalTaskPool;
     }
 
     /**
@@ -754,19 +846,24 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
     //
     //==================================================
 
-    private static Set<Option<?>> OPTIONS = Option.setBuilder()
+    private final static Set<Option<?>> OPTIONS = Option.setBuilder()
             .add(Options.WORKER_TASK_CORE_THREADS)
             .add(Options.WORKER_TASK_MAX_THREADS)
             .add(Options.WORKER_TASK_KEEPALIVE)
             .create();
 
+    private final static Set<Option<?>> EXTERNAL_POOL_OPTIONS = Option.setBuilder()
+            .create();
+
     public boolean supportsOption(final Option<?> option) {
-        return OPTIONS.contains(option);
+        return taskPool instanceof ExternalTaskPool ? EXTERNAL_POOL_OPTIONS.contains(option) : OPTIONS.contains(option);
     }
 
     public <T> T getOption(final Option<T> option) throws IOException {
-        if (option.equals(Options.WORKER_TASK_CORE_THREADS)) {
-            return option.cast(Integer.valueOf(coreSize));
+        if (! supportsOption(option)) {
+            return null;
+        } else if (option.equals(Options.WORKER_TASK_CORE_THREADS)) {
+            return option.cast(Integer.valueOf(taskPool.getCorePoolSize()));
         } else if (option.equals(Options.WORKER_TASK_MAX_THREADS)) {
             return option.cast(Integer.valueOf(taskPool.getMaximumPoolSize()));
         } else if (option.equals(Options.WORKER_TASK_KEEPALIVE)) {
@@ -777,11 +874,14 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
     }
 
     public <T> T setOption(final Option<T> option, final T value) throws IllegalArgumentException, IOException {
-        if (option.equals(Options.WORKER_TASK_CORE_THREADS)) {
-            return option.cast(Integer.valueOf(coreSizeUpdater.getAndSet(this, Options.WORKER_TASK_CORE_THREADS.cast(value).intValue())));
+        if (! supportsOption(option)) {
+            return null;
+        } else if (option.equals(Options.WORKER_TASK_CORE_THREADS)) {
+            final int old = taskPool.getCorePoolSize();
+            taskPool.setCorePoolSize(Options.WORKER_TASK_CORE_THREADS.cast(value).intValue());
+            return option.cast(Integer.valueOf(old));
         } else if (option.equals(Options.WORKER_TASK_MAX_THREADS)) {
             final int old = taskPool.getMaximumPoolSize();
-            taskPool.setCorePoolSize(Options.WORKER_TASK_MAX_THREADS.cast(value).intValue());
             taskPool.setMaximumPoolSize(Options.WORKER_TASK_MAX_THREADS.cast(value).intValue());
             return option.cast(Integer.valueOf(old));
         } else if (option.equals(Options.WORKER_TASK_KEEPALIVE)) {
@@ -836,7 +936,16 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
      * @return the core worker pool size
      */
     protected final int getCoreWorkerPoolSize() {
-        return coreSize;
+        return taskPool.getCorePoolSize();
+    }
+
+    /**
+     * Get an estimate of the number of busy threads in the worker pool.
+     *
+     * @return the estimated number of busy threads in the worker pool
+     */
+    protected final int getBusyWorkerThreadCount() {
+        return taskPool.getActiveCount();
     }
 
     /**
@@ -854,20 +963,221 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
      * @return the estimated number of tasks
      */
     protected final int getWorkerQueueSize() {
-        return taskQueue.size();
+        return taskPool.getQueueSize();
     }
 
-    final class TaskPool extends ThreadPoolExecutor {
+    //==================================================
+    //
+    // Source address
+    //
+    //==================================================
 
-        TaskPool(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final TimeUnit unit, final BlockingQueue<Runnable> workQueue, final ThreadFactory threadFactory, final RejectedExecutionHandler handler) {
-            super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory, handler);
+    /**
+     * Get the bind address table.
+     *
+     * @return the bind address table
+     */
+    protected CidrAddressTable<InetSocketAddress> getBindAddressTable() {
+        return bindAddressTable;
+    }
+
+    /**
+     * Get the expected bind address for the given destination, if any.
+     *
+     * @return the expected bind address for the given destination, or {@code null} if no explicit bind will be done
+     */
+    public InetSocketAddress getBindAddress(InetAddress destination) {
+        return bindAddressTable.get(destination);
+    }
+
+    //==================================================
+    //
+    // JMX
+    //
+    //==================================================
+
+    public abstract XnioWorkerMXBean getMXBean();
+
+    protected abstract ManagementRegistration registerServerMXBean(XnioServerMXBean metrics);
+
+    //==================================================
+    //
+    // Builder
+    //
+    //==================================================
+
+    /**
+     * A builder which allows workers to be programmatically configured.
+     */
+    public static class Builder {
+        private final Xnio xnio;
+        private ExecutorService externalExecutorService;
+        private Runnable terminationTask;
+        private String workerName;
+        private int coreWorkerPoolSize = 4;
+        private int maxWorkerPoolSize = 16;
+        private ThreadGroup threadGroup;
+        private boolean daemon;
+        private int workerKeepAlive = 60_000;
+        private int workerIoThreads = 1;
+        private long workerStackSize = 0L;
+        private CidrAddressTable<InetSocketAddress> bindAddressConfigurations = new CidrAddressTable<>();
+
+        /**
+         * Construct a new instance.
+         *
+         * @param xnio the XNIO instance (must not be {@code null})
+         */
+        protected Builder(final Xnio xnio) {
+            this.xnio = xnio;
         }
 
-        protected void terminated() {
-            taskPoolTerminated();
+        public Xnio getXnio() {
+            return xnio;
+        }
+
+        public Builder populateFromOptions(OptionMap optionMap) {
+            setWorkerName(optionMap.get(Options.WORKER_NAME));
+            setCoreWorkerPoolSize(optionMap.get(Options.WORKER_TASK_CORE_THREADS, coreWorkerPoolSize));
+            setMaxWorkerPoolSize(optionMap.get(Options.WORKER_TASK_MAX_THREADS, maxWorkerPoolSize));
+            setDaemon(optionMap.get(Options.THREAD_DAEMON, daemon));
+            setWorkerKeepAlive(optionMap.get(Options.WORKER_TASK_KEEPALIVE, workerKeepAlive));
+            if (optionMap.contains(Options.WORKER_IO_THREADS)) {
+                setWorkerIoThreads(optionMap.get(Options.WORKER_IO_THREADS, 1));
+            } else if (optionMap.contains(Options.WORKER_READ_THREADS) || optionMap.contains(Options.WORKER_WRITE_THREADS)) {
+                setWorkerIoThreads(max(optionMap.get(Options.WORKER_READ_THREADS, 1), optionMap.get(Options.WORKER_WRITE_THREADS, 1)));
+            }
+            setWorkerStackSize(optionMap.get(Options.STACK_SIZE, workerStackSize));
+            return this;
+        }
+
+        public Builder addBindAddressConfiguration(CidrAddress cidrAddress, InetAddress bindAddress) {
+            return addBindAddressConfiguration(cidrAddress, new InetSocketAddress(bindAddress, 0));
+        }
+
+        public Builder addBindAddressConfiguration(CidrAddress cidrAddress, InetSocketAddress bindAddress) {
+            if (cidrAddress.getNetworkAddress().getClass() != bindAddress.getAddress().getClass()) {
+                throw Messages.msg.mismatchAddressType(cidrAddress.getNetworkAddress().getClass(), bindAddress.getAddress().getClass());
+            }
+            bindAddressConfigurations.put(cidrAddress, bindAddress);
+            return this;
+        }
+
+        public Builder setBindAddressConfigurations(CidrAddressTable<InetSocketAddress> newTable) {
+            bindAddressConfigurations = newTable;
+            return this;
+        }
+
+        public CidrAddressTable<InetSocketAddress> getBindAddressConfigurations() {
+            return bindAddressConfigurations;
+        }
+
+        public Runnable getTerminationTask() {
+            return terminationTask;
+        }
+
+        public Builder setTerminationTask(final Runnable terminationTask) {
+            this.terminationTask = terminationTask;
+            return this;
+        }
+
+        public String getWorkerName() {
+            return workerName;
+        }
+
+        public Builder setWorkerName(final String workerName) {
+            this.workerName = workerName;
+            return this;
+        }
+
+        public int getCoreWorkerPoolSize() {
+            return coreWorkerPoolSize;
+        }
+
+        public Builder setCoreWorkerPoolSize(final int coreWorkerPoolSize) {
+            Assert.checkMinimumParameter("coreWorkerPoolSize", 0, coreWorkerPoolSize);
+            this.coreWorkerPoolSize = coreWorkerPoolSize;
+            return this;
+        }
+
+        public int getMaxWorkerPoolSize() {
+            return maxWorkerPoolSize;
+        }
+
+        public Builder setMaxWorkerPoolSize(final int maxWorkerPoolSize) {
+            Assert.checkMinimumParameter("maxWorkerPoolSize", 0, maxWorkerPoolSize);
+            this.maxWorkerPoolSize = maxWorkerPoolSize;
+            return this;
+        }
+
+        public ThreadGroup getThreadGroup() {
+            return threadGroup;
+        }
+
+        public Builder setThreadGroup(final ThreadGroup threadGroup) {
+            this.threadGroup = threadGroup;
+            return this;
+        }
+
+        public boolean isDaemon() {
+            return daemon;
+        }
+
+        public Builder setDaemon(final boolean daemon) {
+            this.daemon = daemon;
+            return this;
+        }
+
+        public long getWorkerKeepAlive() {
+            return workerKeepAlive;
+        }
+
+        public Builder setWorkerKeepAlive(final int workerKeepAlive) {
+            Assert.checkMinimumParameter("workerKeepAlive", 0, workerKeepAlive);
+            this.workerKeepAlive = workerKeepAlive;
+            return this;
+        }
+
+        public int getWorkerIoThreads() {
+            return workerIoThreads;
+        }
+
+        public Builder setWorkerIoThreads(final int workerIoThreads) {
+            Assert.checkMinimumParameter("workerIoThreads", 0, workerIoThreads);
+            this.workerIoThreads = workerIoThreads;
+            return this;
+        }
+
+        public long getWorkerStackSize() {
+            return workerStackSize;
+        }
+
+        public Builder setWorkerStackSize(final long workerStackSize) {
+            Assert.checkMinimumParameter("workerStackSize", 0, workerStackSize);
+            this.workerStackSize = workerStackSize;
+            return this;
+        }
+
+        public ExecutorService getExternalExecutorService() {
+            return externalExecutorService;
+        }
+
+        public Builder setExternalExecutorService(final ExecutorService executorService) {
+            this.externalExecutorService = executorService;
+            return this;
+        }
+
+        public XnioWorker build() {
+            return xnio.build(this);
         }
     }
 
+    //==================================================
+    //
+    // Private
+    //
+    //==================================================
+
     static class StreamConnectionWrapListener implements ChannelListener<StreamConnection> {
 
         private final FutureResult<ConnectedStreamChannel> futureResult;
@@ -931,19 +1241,19 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
     class WorkerThreadFactory implements ThreadFactory {
 
         private final ThreadGroup threadGroup;
-        private final OptionMap optionMap;
+        private final long stackSize;
         private final boolean markThreadAsDaemon;
 
-        WorkerThreadFactory(final ThreadGroup threadGroup, final OptionMap optionMap, final boolean markThreadAsDaemon) {
+        WorkerThreadFactory(final ThreadGroup threadGroup, final long stackSize, final boolean markThreadAsDaemon) {
             this.threadGroup = threadGroup;
-            this.optionMap = optionMap;
+            this.stackSize = stackSize;
             this.markThreadAsDaemon = markThreadAsDaemon;
         }
 
         public Thread newThread(final Runnable r) {
             return doPrivileged(new PrivilegedAction<Thread>() {
                 public Thread run() {
-                    final Thread taskThread = new Thread(threadGroup, r, name + " task-" + getNextSeq(), optionMap.get(Options.STACK_SIZE, 0L));
+                    final Thread taskThread = new Thread(threadGroup, r, name + " task-" + getNextSeq(), stackSize);
                     // Mark the thread as daemon if the Options.THREAD_DAEMON has been set
                     if (markThreadAsDaemon) {
                         taskThread.setDaemon(true);
@@ -953,5 +1263,162 @@ public abstract class XnioWorker extends AbstractExecutorService implements Conf
             });
         }
     }
+
+    interface TaskPool {
+
+        void shutdown();
+
+        List<Runnable> shutdownNow();
+
+        void execute(Runnable command);
+
+        int getCorePoolSize();
+
+        int getMaximumPoolSize();
+
+        long getKeepAliveTime(TimeUnit unit);
+
+        void setCorePoolSize(int size);
+
+        void setMaximumPoolSize(int size);
+
+        void setKeepAliveTime(long time, TimeUnit unit);
+
+        int getActiveCount();
+
+        int getQueueSize();
+    }
+
+    static class ThreadPoolExecutorTaskPool extends ThreadPoolExecutor implements TaskPool {
+        private final Runnable terminationTask;
+
+        ThreadPoolExecutorTaskPool(final int corePoolSize, final int maximumPoolSize, final long keepAliveTime, final TimeUnit unit, final BlockingQueue<Runnable> workQueue, final ThreadFactory threadFactory, final Runnable terminationTask) {
+            super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue, threadFactory);
+            this.terminationTask = terminationTask;
+        }
+
+        protected void terminated() {
+            terminationTask.run();
+        }
+
+        public void setCorePoolSize(final int size) {
+            setMaximumPoolSize(size);
+        }
+
+        public void setMaximumPoolSize(final int size) {
+            if (size > getCorePoolSize()) {
+                super.setMaximumPoolSize(size);
+                super.setCorePoolSize(size);
+            } else {
+                super.setCorePoolSize(size);
+                super.setMaximumPoolSize(size);
+            }
+        }
+
+        public int getQueueSize() {
+            return getQueue().size();
+        }
+    }
+
+    static class EnhancedQueueExecutorTaskPool implements TaskPool {
+        private final EnhancedQueueExecutor executor;
+
+        EnhancedQueueExecutorTaskPool(final EnhancedQueueExecutor executor) {
+            this.executor = executor;
+        }
+
+        public void shutdown() {
+            executor.shutdown();
+        }
+
+        public List<Runnable> shutdownNow() {
+            return executor.shutdownNow();
+        }
+
+        public void execute(final Runnable command) {
+            executor.execute(command);
+        }
+
+        public int getCorePoolSize() {
+            return executor.getCorePoolSize();
+        }
+
+        public int getMaximumPoolSize() {
+            return executor.getMaximumPoolSize();
+        }
+
+        public long getKeepAliveTime(final TimeUnit unit) {
+            return executor.getKeepAliveTime(unit);
+        }
+
+        public void setCorePoolSize(final int size) {
+            executor.setCorePoolSize(size);
+        }
+
+        public void setMaximumPoolSize(final int size) {
+            executor.setMaximumPoolSize(size);
+        }
+
+        public void setKeepAliveTime(final long time, final TimeUnit unit) {
+            executor.setKeepAliveTime(time, unit);
+        }
+
+        public int getActiveCount() {
+            return executor.getActiveCount();
+        }
+
+        public int getQueueSize() {
+            return executor.getQueueSize();
+        }
+    }
+
+    static class ExternalTaskPool implements TaskPool {
+        private final ExecutorService delegate;
+
+        ExternalTaskPool(final ExecutorService delegate) {
+            this.delegate = delegate;
+        }
+
+        public void shutdown() {
+            // no operation
+        }
+
+        public List<Runnable> shutdownNow() {
+            return Collections.emptyList();
+        }
+
+        public void execute(final Runnable command) {
+            delegate.execute(command);
+        }
+
+        public int getCorePoolSize() {
+            return -1;
+        }
+
+        public int getMaximumPoolSize() {
+            return -1;
+        }
+
+        public long getKeepAliveTime(final TimeUnit unit) {
+            return -1;
+        }
+
+        public void setCorePoolSize(final int size) {
+        }
+
+        public void setMaximumPoolSize(final int size) {
+        }
+
+        public void setKeepAliveTime(final long time, final TimeUnit unit) {
+        }
+
+        public int getActiveCount() {
+            return -1;
+        }
+
+        public int getQueueSize() {
+            return -1;
+        }
+    }
 }
 
