@@ -65,7 +65,7 @@ public class ConnectedSslStreamChannelWriteTestCase extends AbstractConnectedSsl
         assertFalse(buffer.hasRemaining());
         // channel should not be able to shutdown writes... for that, it must receive a close message
         sslChannel.shutdownWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sslChannel.flush());
+        assertFalse(sslChannel.flush());
         // send the close message
         conduitMock.setReadData(CLOSE_MSG);
         conduitMock.enableReads(true);
@@ -92,7 +92,7 @@ public class ConnectedSslStreamChannelWriteTestCase extends AbstractConnectedSsl
         assertFalse(buffer.hasRemaining());
         // channel should not be able to shutdown writes... for that, it must receive a close message
         sslChannel.shutdownWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sslChannel.flush());
+        assertFalse(sslChannel.flush());
         // send the close message
         conduitMock.setReadData(CLOSE_MSG);
         conduitMock.enableReads(true);
@@ -209,7 +209,7 @@ public class ConnectedSslStreamChannelWriteTestCase extends AbstractConnectedSsl
 
         // channel should not be able to shutdown writes... for that, it must receive a close message
         sslChannel.shutdownWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sslChannel.flush());
+        assertFalse(sslChannel.flush());
         // send the close message
         conduitMock.setReadData("{message closed}");
         conduitMock.enableReads(true);
@@ -236,9 +236,12 @@ public class ConnectedSslStreamChannelWriteTestCase extends AbstractConnectedSsl
         assertEquals(0, sslChannel.write(buffer));
         // make sure that channel managed to do the WRAP and got stalled on NEED_UNWRAP handshake status
         assertSame(HandshakeStatus.NEED_UNWRAP, engineMock.getHandshakeStatus());
-        // channel should not be able to shutdown writes... for that, it must receive a close message
+        // channel should not be able to shutdown writes... for that, it must receive a handshake message
+        sslChannel.shutdownWrites();
+        assertFalse(sslChannel.flush());
+        conduitMock.setReadData(HANDSHAKE_MSG);
+        conduitMock.enableReads(true);
         sslChannel.shutdownWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sslChannel.flush());
         // close channel
         sslChannel.close();
         // data expected to have been written to 'conduitMock' by 'sslChannel'
@@ -429,12 +432,12 @@ public class ConnectedSslStreamChannelWriteTestCase extends AbstractConnectedSsl
         }
     }
 
-    // FIXME @Test
+    @Test
     public void closeWithoutFlushing() throws IOException {
         ByteBuffer buffer = ByteBuffer.allocate(10);
         buffer.put("abc".getBytes("UTF-8")).flip();
         sslChannel.write(buffer);
-        sslChannel.close();
-        assertWrittenMessage("abc");
+        sslChannel.shutdownReads();
+        assertWrittenMessage("abc", CLOSE_MSG);
     }
 }
