@@ -21,6 +21,7 @@ package org.xnio.ssl;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
+import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 import static org.xnio.ssl.mock.SSLEngineMock.CLOSE_MSG;
@@ -32,6 +33,7 @@ import static org.xnio.ssl.mock.SSLEngineMock.HandshakeAction.NEED_UNWRAP;
 import static org.xnio.ssl.mock.SSLEngineMock.HandshakeAction.NEED_WRAP;
 import static org.xnio.ssl.mock.SSLEngineMock.HandshakeAction.PERFORM_REQUESTED_ACTION;
 
+import java.io.EOFException;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 
@@ -62,7 +64,7 @@ public class JsseSslStreamSinkConduitTestCase extends AbstractSslConnectionTest
         assertFalse(buffer.hasRemaining());
         // conduit should not be able to shutdown writes... for that, it must receive a close message
         sinkConduit.terminateWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sinkConduit.flush());
+        assertFalse(sinkConduit.flush());
         // send the close message
         conduitMock.setReadData(CLOSE_MSG);
         conduitMock.enableReads(true);
@@ -89,7 +91,7 @@ public class JsseSslStreamSinkConduitTestCase extends AbstractSslConnectionTest
         assertFalse(buffer.hasRemaining());
         // conduit should not be able to shutdown writes... for that, it must receive a close message
         sinkConduit.terminateWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sinkConduit.flush());
+        assertFalse(sinkConduit.flush());
         // send the close message
         conduitMock.setReadData(CLOSE_MSG);
         conduitMock.enableReads(true);
@@ -206,7 +208,7 @@ public class JsseSslStreamSinkConduitTestCase extends AbstractSslConnectionTest
 
         // conduit should not be able to terminate writes... for that, it must receive a close message
         sinkConduit.terminateWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sinkConduit.flush());
+        assertFalse(sinkConduit.flush());
         // send the close message
         conduitMock.setReadData("{message closed}");
         conduitMock.enableReads(true);
@@ -235,7 +237,9 @@ public class JsseSslStreamSinkConduitTestCase extends AbstractSslConnectionTest
         assertSame(HandshakeStatus.NEED_UNWRAP, engineMock.getHandshakeStatus());
         // conduit should not be able to shutdown writes... for that, it must receive a close message
         sinkConduit.terminateWrites();
-        // FIXME workaround for bug found in SSLEngine assertFalse(sinkConduit.flush());
+        assertFalse(sinkConduit.flush());
+        conduitMock.setReadData(HANDSHAKE_MSG);
+        conduitMock.enableReads(true);
         // close connection
         sourceConduit.terminateReads();
         // data expected to have been written to 'conduitMock' by 'sinkConduit'
@@ -432,6 +436,6 @@ public class JsseSslStreamSinkConduitTestCase extends AbstractSslConnectionTest
         buffer.put("abc".getBytes("UTF-8")).flip();
         sinkConduit.write(buffer);
         sourceConduit.terminateReads();
-        assertWrittenMessage("abc");
+        assertWrittenMessage("abc", CLOSE_MSG);
     }
 }
