@@ -45,6 +45,7 @@ import org.xnio.Options;
 import org.xnio.Pool;
 import org.xnio.channels.AssembledConnectedSslStreamChannel;
 import org.xnio.channels.ConnectedSslStreamChannel;
+import org.xnio.channels.ConnectedStreamChannel;
 import org.xnio.channels.SslChannel;
 import org.xnio.ssl.mock.SSLEngineMock.HandshakeAction;
 
@@ -55,14 +56,18 @@ import org.xnio.ssl.mock.SSLEngineMock.HandshakeAction;
  */
 public class StartTLSChannelTestCase extends AbstractConnectedSslStreamChannelTest {
 
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({ "unchecked", "rawtypes" })
     @Override
     protected ConnectedSslStreamChannel createSslChannel() {
         final Pool<ByteBuffer> socketBufferPool = new ByteBufferSlicePool(BufferAllocator.BYTE_BUFFER_ALLOCATOR, 17000, 17000 * 16);
         final Pool<ByteBuffer> applicationBufferPool = new ByteBufferSlicePool(BufferAllocator.BYTE_BUFFER_ALLOCATOR, 17000, 17000 * 16);
-        SslConnection connection = new JsseSslStreamConnection(connectionMock, engineMock, socketBufferPool, applicationBufferPool, true);
+        final SslConnection connection = new JsseSslConnection(connectionMock, engineMock, socketBufferPool, applicationBufferPool);
         final ConnectedSslStreamChannel channel = new AssembledConnectedSslStreamChannel(connection, connection.getSourceChannel(), connection.getSinkChannel());
-        channel.getReadSetter().set(context.mock(ChannelListener.class, "read listener"));
+        final ChannelListener readListener = context.mock(ChannelListener.class, "read listener");
+        channel.getReadSetter().set(readListener);
+        context.checking(new Expectations() {{
+            allowing(readListener).handleEvent(with(any(ConnectedStreamChannel.class)));
+        }});
         channel.getWriteSetter().set(context.mock(ChannelListener.class, "write listener"));
         return channel;
     }
@@ -128,7 +133,7 @@ public class StartTLSChannelTestCase extends AbstractConnectedSslStreamChannelTe
         // attempt to read... channel is expected to return 0 as it stumbles upon a NEED_WRAP that cannot be executed
         assertEquals(0, sslChannel.read(new ByteBuffer[]{buffer}));
         // everything is kept the same
-        assertTrue(conduitMock.isReadResumed());
+        assertFalse(conduitMock.isReadResumed());
         assertTrue(sslChannel.isReadResumed());
         assertFalse(conduitMock.isWriteResumed());
         assertFalse(sslChannel.isWriteResumed());
@@ -278,7 +283,7 @@ public class StartTLSChannelTestCase extends AbstractConnectedSslStreamChannelTe
         sslChannel.resumeReads();
         assertTrue(sslChannel.isReadResumed());
         assertFalse(conduitMock.isReadAwaken());
-        assertTrue(conduitMock.isReadResumed());
+        assertFalse(conduitMock.isReadResumed());
         sslChannel.suspendReads();
         assertFalse(sslChannel.isReadResumed());
         assertFalse(conduitMock.isReadResumed());
@@ -407,15 +412,25 @@ public class StartTLSChannelTestCase extends AbstractConnectedSslStreamChannelTe
         assertFalse(sslChannel.flush());
         assertFalse(conduitMock.isWriteShutdown());
         conduitMock.enableWrites(true);
-        // FIXME workaround for bug found in SSLEngine assertFalse(sslChannel.flush());
-        assertFalse(conduitMock.isWriteShutdown());
+
+        /* One might assume that this flush should return false, since the close message has not yet been
+           received; however, this is not part of the write shutdown contract.  Thus as soon as we flush our own
+           write termination message to the wire, we consider writes to be fully shut down, and that's that.  It
+           is up to the reader side to make sure that read is carried through to EOF before terminating reads.  If
+           a read termination is received before flushing the write shutdown, then shutdown will close the channel.
+           If the read termination comes after, then upon reading the -1, when reads are terminated, the channel will
+           close at that time. */
+        assertTrue(sslChannel.flush());
+        assertTrue(sslChannel.flush());
 
         conduitMock.setReadData(CLOSE_MSG);
+        buffer.clear();
+        assertEquals(-1, sslChannel.read(buffer));
+        sslChannel.shutdownReads();
         assertTrue(sslChannel.flush());
+        assertTrue(conduitMock.isReadShutdown());
         assertTrue(conduitMock.isWriteShutdown());
 
-        assertTrue(conduitMock.isOpen());
-        sslChannel.close();
         assertFalse(conduitMock.isOpen());
 
         assertWrittenMessage("MSG", CLOSE_MSG);
@@ -426,7 +441,7 @@ public class StartTLSChannelTestCase extends AbstractConnectedSslStreamChannelTe
     @Test
     public void startTLSWithReadNeedsWrap() throws IOException {
         //set a handshake listener
-        final ChannelListener<SslChannel> listener = context.mock(ChannelListener.class, "read needs unwrap");
+        final ChannelListener<SslChannel> listener = context.mock(ChannelListener.class, "read needs wrap");
         context.checking(new Expectations() {{
             atLeast(1).of(listener).handleEvent(sslChannel);
         }});
@@ -445,30 +460,28 @@ public class StartTLSChannelTestCase extends AbstractConnectedSslStreamChannelTe
         // start tls
         sslChannel.startHandshake();
 
-        // attempt to write... channel is expected to return 0 as it stumbles upon a NEED_UNWRAP that cannot be executed
-        assertEquals(0, sslChannel.read(buffer));
-        assertFalse(conduitMock.isReadResumed());
-        assertFalse(conduitMock.isWriteResumed());
-        assertSame(HandshakeStatus.FINISHED, engineMock.getHandshakeStatus());
-        conduitMock.enableWrites(true);
+        // attempt to write... channel is expected to return 3 regardless of the fact it cannot flush
+        assertSame(HandshakeStatus.NEED_WRAP, engineMock.getHandshakeStatus());
         assertEquals(3, sslChannel.read(new ByteBuffer[]{buffer}));
         assertReadMessage(buffer, "MSG");
         assertSame(HandshakeStatus.NOT_HANDSHAKING, engineMock.getHandshakeStatus());
-        assertWrittenMessage(HANDSHAKE_MSG);
+        assertWrittenMessage("");
 
+        assertFalse(sslChannel.flush());
+        // enable writes and hence flush handshake message
+        conduitMock.enableWrites(true);
         assertTrue(sslChannel.flush());
         assertWrittenMessage(HANDSHAKE_MSG);
 
         sslChannel.shutdownReads();
+
         assertFalse(conduitMock.isWriteShutdown());
         conduitMock.setReadData(CLOSE_MSG);
         sslChannel.shutdownWrites();
         assertTrue(sslChannel.flush());
         assertTrue(conduitMock.isWriteShutdown());
 
-        // channel not yet closed
-        assertTrue(conduitMock.isOpen());
-        sslChannel.close();
+        // channel closed
         assertFalse(conduitMock.isOpen());
 
         assertWrittenMessage(HANDSHAKE_MSG, CLOSE_MSG);
