@@ -18,8 +18,12 @@
 
 package org.xnio.nio;
 
-import java.io.Closeable;
+import static org.xnio.IoUtils.safeClose;
+import static org.xnio.nio.Log.log;
+import static org.xnio.nio.Log.tcpServerLog;
+
 import java.io.IOException;
+import java.net.InetSocketAddress;
 import java.net.ServerSocket;
 import java.net.Socket;
 import java.net.SocketAddress;
@@ -32,10 +36,13 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
 import java.util.concurrent.atomic.AtomicLongFieldUpdater;
+
 import org.jboss.logging.Logger;
-import org.xnio.IoUtils;
-import org.xnio.Option;
 import org.xnio.ChannelListener;
+import org.xnio.ManagementRegistration;
+import org.xnio.IoUtils;
+import org.xnio.LocalSocketAddress;
+import org.xnio.Option;
 import org.xnio.OptionMap;
 import org.xnio.Options;
 import org.xnio.StreamConnection;
@@ -45,10 +52,6 @@ import org.xnio.channels.AcceptingChannel;
 import org.xnio.channels.UnsupportedOptionException;
 import org.xnio.management.XnioServerMXBean;
 
-import static org.xnio.IoUtils.safeClose;
-import static org.xnio.nio.Log.log;
-import static org.xnio.nio.Log.tcpServerLog;
-
 final class NioTcpServer extends AbstractNioChannel<NioTcpServer> implements AcceptingChannel<StreamConnection>, AcceptListenerSettable<NioTcpServer> {
     private static final String FQCN = NioTcpServer.class.getName();
 
@@ -58,7 +61,7 @@ final class NioTcpServer extends AbstractNioChannel<NioTcpServer> implements Acc
 
     private final ServerSocketChannel channel;
     private final ServerSocket socket;
-    private final Closeable mbeanHandle;
+    private final ManagementRegistration mbeanHandle;
 
     private static final Set<Option<?>> options = Option.setBuilder()
             .add(Options.REUSE_ADDRESSES)
@@ -186,46 +189,47 @@ final class NioTcpServer extends AbstractNioChannel<NioTcpServer> implements Acc
                 handles[i].initializeTokenCount(i < tokens ? connections : 0);
             }
         }
-        mbeanHandle = NioXnio.register(new XnioServerMXBean() {
-            public String getProviderName() {
-                return "nio";
-            }
+        mbeanHandle = worker.registerServerMXBean(
+                new XnioServerMXBean() {
+                    public String getProviderName() {
+                        return "nio";
+                    }
 
-            public String getWorkerName() {
-                return worker.getName();
-            }
+                    public String getWorkerName() {
+                        return worker.getName();
+                    }
 
-            public String getBindAddress() {
-                return String.valueOf(getLocalAddress());
-            }
+                    public String getBindAddress() {
+                        return String.valueOf(getLocalAddress());
+                    }
 
-            public int getConnectionCount() {
-                final AtomicInteger counter = new AtomicInteger();
-                final CountDownLatch latch = new CountDownLatch(handles.length);
-                for (final NioTcpServerHandle handle : handles) {
-                    handle.getWorkerThread().execute(new Runnable() {
-                        public void run() {
-                            counter.getAndAdd(handle.getConnectionCount());
-                            latch.countDown();
+                    public int getConnectionCount() {
+                        final AtomicInteger counter = new AtomicInteger();
+                        final CountDownLatch latch = new CountDownLatch(handles.length);
+                        for (final NioTcpServerHandle handle : handles) {
+                            handle.getWorkerThread().execute(() -> {
+                                counter.getAndAdd(handle.getConnectionCount());
+                                latch.countDown();
+                            });
                         }
-                    });
-                }
-                try {
-                    latch.await();
-                } catch (InterruptedException e) {
-                    Thread.currentThread().interrupt();
-                }
-                return counter.get();
-            }
+                        try {
+                            latch.await();
+                        } catch (InterruptedException e) {
+                            Thread.currentThread().interrupt();
+                        }
+                        return counter.get();
+                    }
 
-            public int getConnectionLimitHighWater() {
-                return getHighWater(connectionStatus);
-            }
+                    public int getConnectionLimitHighWater() {
+                        return getHighWater(connectionStatus);
+                    }
+
+                    public int getConnectionLimitLowWater() {
+                        return getLowWater(connectionStatus);
+                    }
+                }
+        );
 
-            public int getConnectionLimitLowWater() {
-                return getLowWater(connectionStatus);
-            }
-        });
     }
 
     private static IllegalArgumentException badLowWater(final int highWater) {
@@ -241,7 +245,7 @@ final class NioTcpServer extends AbstractNioChannel<NioTcpServer> implements Acc
             channel.close();
         } finally {
             for (NioTcpServerHandle handle : handles) {
-                handle.getWorkerThread().cancelKey(handle.getSelectionKey());
+                handle.cancelKey(false);
             }
             safeClose(mbeanHandle);
         }
@@ -384,6 +388,25 @@ final class NioTcpServer extends AbstractNioChannel<NioTcpServer> implements Acc
         try {
             accepted = channel.accept();
             if (accepted != null) try {
+                final SocketAddress localAddress = accepted.getLocalAddress();
+                int hash;
+                if (localAddress instanceof InetSocketAddress) {
+                    final InetSocketAddress address = (InetSocketAddress) localAddress;
+                    hash = address.getAddress().hashCode() * 23 + address.getPort();
+                } else if (localAddress instanceof LocalSocketAddress) {
+                    hash = ((LocalSocketAddress) localAddress).getName().hashCode();
+                } else {
+                    hash = localAddress.hashCode();
+                }
+                final SocketAddress remoteAddress = accepted.getRemoteAddress();
+                if (remoteAddress instanceof InetSocketAddress) {
+                    final InetSocketAddress address = (InetSocketAddress) remoteAddress;
+                    hash = (address.getAddress().hashCode() * 23 + address.getPort()) * 23 + hash;
+                } else if (remoteAddress instanceof LocalSocketAddress) {
+                    hash = ((LocalSocketAddress) remoteAddress).getName().hashCode() * 23 + hash;
+                } else {
+                    hash = localAddress.hashCode() * 23 + hash;
+                }
                 accepted.configureBlocking(false);
                 final Socket socket = accepted.socket();
                 socket.setKeepAlive(keepAlive != 0);
@@ -391,8 +414,9 @@ final class NioTcpServer extends AbstractNioChannel<NioTcpServer> implements Acc
                 socket.setTcpNoDelay(tcpNoDelay != 0);
                 final int sendBuffer = this.sendBuffer;
                 if (sendBuffer > 0) socket.setSendBufferSize(sendBuffer);
-                final SelectionKey selectionKey = current.registerChannel(accepted);
-                final NioSocketStreamConnection newConnection = new NioSocketStreamConnection(current, selectionKey, handle);
+                final WorkerThread ioThread = worker.getIoThread(hash);
+                final SelectionKey selectionKey = ioThread.registerChannel(accepted);
+                final NioSocketStreamConnection newConnection = new NioSocketStreamConnection(ioThread, selectionKey, handle);
                 newConnection.setOption(Options.READ_TIMEOUT, Integer.valueOf(readTimeout));
                 newConnection.setOption(Options.WRITE_TIMEOUT, Integer.valueOf(writeTimeout));
                 ok = true;
