@@ -248,7 +248,7 @@ final class QueuedNioTcpServer extends AbstractNioChannel<QueuedNioTcpServer> im
         try {
             channel.close();
         } finally {
-            handle.getWorkerThread().cancelKey(handle.getSelectionKey());
+            handle.cancelKey(true);
             safeClose(mbeanHandle);
         }
     }
@@ -357,6 +357,7 @@ final class QueuedNioTcpServer extends AbstractNioChannel<QueuedNioTcpServer> im
                 if(openConnections >= getHighWater(connectionStatus)) {
                     synchronized (QueuedNioTcpServer.this) {
                         suspendedDueToWatermark = true;
+                        tcpServerLog.logf(FQCN, Logger.Level.DEBUG, null, "Total open connections reach high water limit (%s) after updating water mark", getHighWater(connectionStatus));
                     }
                 } else if(suspendedDueToWatermark && openConnections <= getLowWater(connectionStatus)) {
                     suspendedDueToWatermark = false;
@@ -392,14 +393,13 @@ final class QueuedNioTcpServer extends AbstractNioChannel<QueuedNioTcpServer> im
                 ok = true;
                 return newConnection;
             } finally {
-                if (! ok) safeClose(accepted);
+                if (! ok) {
+                    safeClose(accepted);
+                    handle.freeConnection();
+                }
             }
         } catch (IOException e) {
             return null;
-        } finally {
-            if (! ok) {
-                handle.freeConnection();
-            }
         }
         // by contract, only a resume will do
         return null;
@@ -476,12 +476,12 @@ final class QueuedNioTcpServer extends AbstractNioChannel<QueuedNioTcpServer> im
         try {
             accepted = channel.accept();
             if(suspendedDueToWatermark) {
+                tcpServerLog.logf(FQCN, Logger.Level.DEBUG, null, "Exceeding connection high water limit (%s). Closing this new accepting request %s", getHighWater(connectionStatus), accepted);
                 IoUtils.safeClose(accepted);
                 return;
             }
-
         } catch (IOException e) {
-            tcpServerLog.logf(FQCN, Logger.Level.ERROR, e, "Exception accepting request, closing server channel %s", this);
+            tcpServerLog.logf(FQCN, Logger.Level.DEBUG, e, "Exception accepting request, closing server channel %s", this);
             IoUtils.safeClose(channel);
             return;
         }
@@ -525,6 +525,7 @@ final class QueuedNioTcpServer extends AbstractNioChannel<QueuedNioTcpServer> im
                 if(openConnections >= getHighWater(connectionStatus)) {
                     synchronized (QueuedNioTcpServer.this) {
                         suspendedDueToWatermark = true;
+                        tcpServerLog.logf(FQCN, Logger.Level.DEBUG, null, "Total open connections reach high water limit (%s) by this new accepting request %s", getHighWater(connectionStatus), accepted);
                     }
                 }
             } finally {
