@@ -21,6 +21,7 @@ package org.xnio.nio.test;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
+import java.io.EOFException;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.net.Inet4Address;
@@ -36,7 +37,6 @@ import java.util.concurrent.atomic.AtomicInteger;
 import org.jboss.logging.Logger;
 import org.junit.After;
 import org.junit.Before;
-import org.junit.Ignore;
 import org.junit.Test;
 import org.xnio.ChannelListener;
 import org.xnio.ChannelListeners;
@@ -481,7 +481,6 @@ public abstract class AbstractNioTcpTest<T extends ConnectedChannel, R extends S
     }
 
     @Test
-    @Ignore("Does not follow thread model")
     public void twoWayTransfer() throws Exception {
         log.info("Test: twoWayTransfer");
         final CountDownLatch latch = new CountDownLatch(2);
@@ -621,166 +620,4 @@ public abstract class AbstractNioTcpTest<T extends ConnectedChannel, R extends S
         assertEquals(serverSent.get(), clientReceived.get());
         assertEquals(clientSent.get(), serverReceived.get());
     }
-
-    @Test
-    public void clientNastyClose() throws Exception {
-        log.info("Test: clientNastyClose");
-        final CountDownLatch latch = new CountDownLatch(2);
-        final AtomicBoolean clientOK = new AtomicBoolean(false);
-        final AtomicBoolean serverOK = new AtomicBoolean(false);
-        doConnectionTest(new Runnable() {
-            public void run() {
-                try {
-                    assertTrue(latch.await(500L, TimeUnit.MILLISECONDS));
-                } catch (InterruptedException e) {
-                    throw new RuntimeException(e);
-                }
-            }
-        }, new ChannelListener<T>() {
-            public void handleEvent(final T channel) {
-                try {
-                    channel.getCloseSetter().set(new ChannelListener<ConnectedChannel>() {
-                        public void handleEvent(final ConnectedChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    channel.setOption(Options.CLOSE_ABORT, Boolean.TRUE);
-                    channel.close();
-                    clientOK.set(true);
-                } catch (Throwable t) {
-                    log.errorf(t, "Failed to close channel (propagating as RT exception)");
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        }, new ChannelListener<T>() {
-            public void handleEvent(final T channel) {
-                try {
-                    channel.getCloseSetter().set(new ChannelListener<ConnectedChannel>() {
-                        public void handleEvent(final ConnectedChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    setReadListener(channel, new ChannelListener<R>() {
-                        public void handleEvent(final R sourceChannel) {
-                            int res;
-                            try {
-                                res = sourceChannel.read(ByteBuffer.allocate(100));
-                            } catch (IOException e) {
-                                serverOK.set(true);
-                                IoUtils.safeClose(channel);
-                                return;
-                            }
-                            if (res > 0) IoUtils.safeClose(channel);
-                        }
-                    });
-                    resumeReads(channel);
-                } catch (Throwable t) {
-                    log.errorf(t, "Failed to close channel (propagating as RT exception)");
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        });
-        assertTrue(serverOK.get());
-        assertTrue(clientOK.get());
-    }
-
-    @Test
-    public void serverNastyClose() throws Exception {
-        log.info("Test: serverNastyClose");
-        final CountDownLatch latch = new CountDownLatch(2);
-        final AtomicBoolean clientOK = new AtomicBoolean(false);
-        final AtomicBoolean serverOK = new AtomicBoolean(false);
-        final CountDownLatch serverLatch = new CountDownLatch(1);
-        doConnectionTest(new Runnable() {
-            public void run() {
-                try {
-                    assertTrue(latch.await(500L, TimeUnit.MILLISECONDS));
-                } catch (InterruptedException e) {
-                    throw new RuntimeException(e);
-                }
-            }
-        }, new ChannelListener<T>() {
-            public void handleEvent(final T channel) {
-                try {
-                    log.info("Client opened");
-                    channel.getCloseSetter().set(new ChannelListener<ConnectedChannel>() {
-                        public void handleEvent(final ConnectedChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    setReadListener(channel, new ChannelListener<R>() {
-                        public void handleEvent(final R sourceChannel) {
-                            int res;
-                            try {
-                                res = sourceChannel.read(ByteBuffer.allocate(100));
-                            } catch (IOException e) {
-                                clientOK.set(true);
-                                IoUtils.safeClose(channel);
-                                return;
-                            }
-                            if (res > 0) IoUtils.safeClose(channel);
-                        }
-                    });
-                    setWriteListener(channel, new ChannelListener<W>() {
-                        public void handleEvent(final W sinkChannel) {
-                            try {
-                                if (sinkChannel.write(ByteBuffer.wrap(new byte[] { 1 })) > 0) {
-                                    sinkChannel.suspendWrites();
-                                }
-                            } catch (IOException e) {
-                                IoUtils.safeClose(channel);
-                            }
-                        }
-                    });
-                    resumeReads(channel);
-                    resumeWrites(channel);
-                    serverLatch.countDown();
-                } catch (Throwable t) {
-                    log.error("Error occurred on client", t);
-                    try {
-                        channel.close();
-                    } catch (Throwable t2) {
-                        log.error("Error occurred on client (close)", t2);
-                        latch.countDown();
-                        throw new RuntimeException(t);
-                    }
-                    throw new RuntimeException(t);
-                }
-            }
-        }, new ChannelListener<T>() {
-            public void handleEvent(final T channel) {
-                try {
-                    log.info("Server opened");
-                    channel.getCloseSetter().set(new ChannelListener<ConnectedChannel>() {
-                        public void handleEvent(final ConnectedChannel channel) {
-                            latch.countDown();
-                        }
-                    });
-                    setReadListener(channel, new ChannelListener<R>() {
-                        public void handleEvent(final R sourceChannel) {
-                            try {
-                                if (sourceChannel.read(ByteBuffer.allocate(1)) > 0) {
-                                    log.info("Closing connection...");
-                                    channel.setOption(Options.CLOSE_ABORT, Boolean.TRUE);
-                                    channel.close();
-                                    serverOK.set(true);
-                                }
-                            } catch (IOException e) {
-                                IoUtils.safeClose(channel);
-                            }
-                        }
-                    });
-                    resumeReads(channel);
-                } catch (Throwable t) {
-                    log.error("Error occurred on server", t);
-                    latch.countDown();
-                    throw new RuntimeException(t);
-                }
-            }
-        });
-        assertTrue(serverOK.get());
-        assertTrue(clientOK.get());
-    }
 }
